<?php

require_once plugin_dir_path( __DIR__ ) . 'includes/scssphp/scss.inc.php';
use ScssPhp\ScssPhp\Compiler;
use ScssPhp\ScssPhp\Formatter;
use ScssPhp\ScssPhp\SourceMap\SourceMapGenerator;

class Hm_Wp_Scss_Public {

    private $error;
    private $errorTitle;
    private $errorString;

    /**
     * The ID of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $plugin_name    The ID of this plugin.
     */
    private $plugin_name;

    /**
     * The version of this plugin.
     *
     * @since    1.0.0
     * @access   private
     * @var      string    $version    The current version of this plugin.
     */
    private $version;

    /**
     * Initialize the class and set its properties.
     *
     * @since    1.0.0
     * @param      string    $plugin_name       The name of the plugin.
     * @param      string    $version    The version of this plugin.
     */
    public function __construct( $plugin_name, $version ) {

        $this->plugin_name = $plugin_name;
        $this->version = $version;
        $this->error = false;
        $this->errorTitle = 'WP SCSS Compiling Error';
        $this->errorString = " -- FILE: ";
        if(!function_exists('wp_get_current_user'))
        {
            // Prevents errors with Plugabble.php constants
            if ( !defined('AUTH_COOKIE') ) define('AUTH_COOKIE', "C1");
            if ( !defined('SECURE_AUTH_COOKIE') ) define('SECURE_AUTH_COOKIE', "C2");
            if ( !defined('LOGGED_IN_COOKIE') ) define('LOGGED_IN_COOKIE', "C3");

            require_once(ABSPATH . "wp-includes/pluggable.php");
        }

        // Launch compilation
        if(
            !get_option('hm_wp_scss__compilation_time') ||
            get_option('hm_wp_scss__compilation_time') == 'Always' ||
            ( get_option('hm_wp_scss__compilation_time') == 'When SCSS has changed' && $this->scssHasChanged() ) ||
            ( get_option('hm_wp_scss__compilation_time') == 'If Logged In' && is_user_logged_in() ) ||
            ( get_option('hm_wp_scss__compilation_time') == 'If Admin Logged In' && current_user_can( 'edit_pages' ) )
        )
        {
            // Errors handling
            $this->compile();
        }

    }

    function admin_notice__error() {
        $class = 'notice notice-error';
        printf( '<div class="%1$s"><p><strong>%2$s</strong>%3$s</p></div>', esc_attr( $class ), esc_html( $this->errorTitle ), esc_html( $this->error ) );
    }

    /**
     * Register the stylesheets for the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function enqueue_styles() {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Hm_Wp_Scss_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Hm_Wp_Scss_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */

        wp_enqueue_style( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'css/hm-wp-scss-public.css', array(), $this->version, 'all' );

        // Automatic Enqueue stylesheets
        if( get_option('hm_wp_scss__enqueuing') == '1' )
        {
            if( ( $folder_css = get_option('hm_wp_scss__css_location') ) != "" ):
                $path_css 		= get_stylesheet_directory() . $folder_css;

                if( is_dir( $path_css ) )
                {
                    foreach( scandir( $path_css ) as $file )
                    {
                        if( $this->getFileExtension($file) == 'css' )
                            wp_enqueue_style( $file, get_stylesheet_directory_uri() . $folder_css . $file, array(), filemtime($path_css.$file), 'all' );
                    }
                }
            endif;
            // Path is empty
        }
    }

    /**
     * Register the JavaScript for the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function enqueue_scripts() {

        /**
         * This function is provided for demonstration purposes only.
         *
         * An instance of this class should be passed to the run() function
         * defined in Hm_Wp_Scss_Loader as all of the hooks are defined
         * in that particular class.
         *
         * The Hm_Wp_Scss_Loader will then create the relationship
         * between the defined hooks and the functions defined in this
         * class.
         */

        wp_enqueue_script( $this->plugin_name, plugin_dir_url( __FILE__ ) . 'js/hm-wp-scss-public.js', array( 'jquery' ), $this->version, false );

    }

    private function compile() {

        $compiler = new Compiler();

        // CSS Formatter
        $formatter = get_option('hm_wp_scss__compilation_mode');
        if( $formatter == "" )
            $formatter = 'Compressed';
        $formatterClass = "ScssPhp\ScssPhp\Formatter\\$formatter";
        $compiler->setFormatter(new $formatterClass());

        if( ( $option_scss = get_option('hm_wp_scss__scss_location') ) != "" && ( $option_css = get_option('hm_wp_scss__css_location') ) != "" ):

            $path_scss 	= get_stylesheet_directory() . $option_scss;
            $path_css	= get_stylesheet_directory() . $option_css;

            if( is_dir( $path_scss ) && is_dir( $path_css ) )
            {
                foreach( scandir( $path_scss ) as $file )
                {
                    $ext = $this->getFileExtension($file);
                    $filename = $this->getFilename($file, $ext);

                    if( $ext == 'scss' )
                    {
                        $filename = basename( $file, $ext );
                        $fileCssPath = $path_css . $filename . 'css';

                        // Compilation
                        $compiler->setImportPaths($path_scss);

                        // Source Map File Option
                        if( get_option('hm_wp_scss__source_map_file') == '1' )
                        {
                            $fileMapPath = $path_css . $filename . 'css.map';
                            $compiler->setSourceMap(Compiler::SOURCE_MAP_FILE);
                            $compiler->setSourceMapOptions([
                                'sourceMapWriteTo' => $fileMapPath,
                                'sourceMapFilename' => $path_css . $filename . 'css',
                                'sourceMapBasepath' => ABSPATH,
                                'sourceRoot' => '/',
                            ]);
                        }

                        try
                        {
                            $result = $compiler->compile(file_get_contents($path_scss . $file));
                            file_put_contents($fileCssPath, $result);
                        }
                        catch( Exception $e )
                        {
                            $this->error = $this->errorString . $file . ' -- ERROR: ' . $e->getMessage();
                            add_action( 'admin_notices', [$this,'admin_notice__error'] );

                            // Admin Error
                            if(
                                get_option('hm_wp_scss__errors_display') == 'Always' ||
                                ( get_option('hm_wp_scss__errors_display') == 'If Debug Mode of Wordpress is true' && WP_DEBUG ) ||
                                ( get_option('hm_wp_scss__errors_display') ==  'If Logged In' && is_user_logged_in() ) ||
                                ( get_option('hm_wp_scss__errors_display') == 'If Admin Logged In' && current_user_can( 'edit_pages' ) )
                            ) {
                                $errorTitle = $this->errorTitle;
                                $error = $this->error;

                                add_action( 'wp_head', function () use ( $errorTitle, $error ) {
                                    echo '<style type="text/css">'.
                                        'body::before {
									  font-family: Helvetica, monospace;
									  font-size: 14px;
									  white-space: pre;
									  display: block;
									  padding: 1em;
									  margin-bottom: 1em;
									  border-bottom: 2px solid black;
									  font-weight: bold;
									  content: \'' . esc_html($errorTitle) . esc_html($error) . '\'; }' .
                                        '</style>';
                                });
                            }
                        }
                    }
                }
            }
            else
            {
                // Folders don't exist
            }

        endif;
        // Paths are empty
    }

    private function scssHasChanged()
    {
        $path_scss 	= get_stylesheet_directory() . get_option('hm_wp_scss__scss_location');
        $path_css	= get_stylesheet_directory() . get_option('hm_wp_scss__css_location');

        if( is_dir( $path_scss ) && is_dir( $path_css ) )
        {
            foreach( scandir( $path_scss ) as $file )
            {
                $ext = $this->getFileExtension($file);
                $filename = $this->getFilename($file, $ext);

                if( $ext == 'scss' )
                {
                    $filename = basename( $file, $ext );
                    $fileCssPath = $path_css . $filename . 'css';

                    if( !file_exists($path_css . $filename . 'css') )
                        return true;
                    elseif(
                        file_exists($path_css . $filename . 'css') &&
                        filemtime($path_scss . $filename . 'scss') > filemtime($path_css . $filename . 'css')
                    )
                        return true;
                }
            }
        }
        return false;
    }

    private function getFileExtension( $_filePath )
    {
        $path_parts = pathinfo( $_filePath );
        return $path_parts['extension'];
    }
    private function getFilename( $_filePath, $_ext )
    {
        return basename( $_filePath, $_ext );
    }

}
